@@ -12,8 +12,8 @@ from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Union
+from uuid import UUID
 
-import aioredis
 from pydantic import BaseModel
 from pydantic.fields import SHAPE_DEFAULTDICT
 from pydantic.fields import SHAPE_DICT
@@ -25,8 +25,10 @@ from pydantic.fields import SHAPE_SET
 from pydantic.fields import SHAPE_TUPLE
 from pydantic.fields import SHAPE_TUPLE_ELLIPSIS
 from pydantic_aioredis.config import RedisConfig
+from redis import asyncio as aioredis
 
-JSON_DUMP_SHAPES = [
+# JSON_DUMP_SHAPES are object types that are serialized to JSON using json.dumps
+JSON_DUMP_SHAPES = (
     SHAPE_LIST,
     SHAPE_SET,
     SHAPE_MAPPING,
@@ -37,7 +39,11 @@ JSON_DUMP_SHAPES = [
     SHAPE_DICT,
     SHAPE_DEFAULTDICT,
     Enum,
-]
+)
+
+# STR_DUMP_SHAPES are object types that are serialized to strings using str(obj)
+# They are stored in redis as strings and rely on pydantic to deserialize them
+STR_DUMP_SHAPES = (IPv4Address, IPv4Network, IPv6Address, IPv6Network, UUID)
 
 
 class _AbstractStore(BaseModel):
@@ -65,9 +71,15 @@ class _AbstractModel(BaseModel):
     _store: _AbstractStore
     _primary_key_field: str
     _table_name: Optional[str] = None
+    _auto_sync: bool = True
 
-    @staticmethod
-    def json_default(obj: Any) -> str:
+    @classmethod
+    def json_object_hook(cls, obj: dict):
+        """Can be overridden to handle custom json -> object"""
+        return obj
+
+    @classmethod
+    def json_default(cls, obj: Any) -> str:
         """
         JSON serializer for objects not serializable by default json library
         Currently handles: datetimes -> obj.isoformat, ipaddress and ipnetwork -> str
@@ -76,7 +88,7 @@ class _AbstractModel(BaseModel):
         if isinstance(obj, (datetime, date)):
             return obj.isoformat()
 
-        if isinstance(obj, (IPv4Network, IPv4Address, IPv6Network, IPv6Address)):
+        if isinstance(obj, STR_DUMP_SHAPES):
             return str(obj)
 
         raise TypeError("Type %s not serializable" % type(obj))
@@ -90,8 +102,8 @@ class _AbstractModel(BaseModel):
             None - will be converted to the string "None"
             More complex data types will be json dumped.
 
-        The json dumper uses class.json_serial as its default serializer.
-        Users can override json_serial with a custom json serializer if they chose to.
+        The json dumper uses class.json_default as its default serializer.
+        Users can override json_default with a custom json serializer if they chose to.
         Users can override serialze paritally and deserialze partially
         """
         columns = data.keys()
@@ -120,9 +132,9 @@ class _AbstractModel(BaseModel):
             if field not in columns:
                 continue
             if cls.__fields__[field].type_ not in [str, float, int]:
-                data[field] = json.loads(data[field])
+                data[field] = json.loads(data[field], object_hook=cls.json_object_hook)
             if getattr(cls.__fields__[field], "shape", None) in JSON_DUMP_SHAPES:
-                data[field] = json.loads(data[field])
+                data[field] = json.loads(data[field], object_hook=cls.json_object_hook)
             if getattr(cls.__fields__[field], "allow_none", False):
                 if data[field] == "None":
                     data[field] = None
@@ -142,13 +154,6 @@ class _AbstractModel(BaseModel):
         """Insert into the redis store"""
         raise NotImplementedError("insert should be implemented")
 
-    @classmethod
-    async def update(
-        cls, _id: Any, data: Dict[str, Any], life_span_seconds: Optional[int] = None
-    ):  # pragma: no cover
-        """Update an existing key in the redis store"""
-        raise NotImplementedError("update should be implemented")
-
     @classmethod
     async def delete(cls, ids: Union[Any, List[Any]]):  # pragma: no cover
         """Delete a key from the redis store"""
