@@ -1,11 +1,14 @@
 """Module containing the model classes"""
+import asyncio
+from functools import lru_cache
+from sys import version_info
 from typing import Any
-from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Tuple
 from typing import Union
 
+import nest_asyncio
 from pydantic_aioredis.abstract import _AbstractModel
 from pydantic_aioredis.utils import bytes_to_string
 
@@ -13,25 +16,58 @@ from pydantic_aioredis.utils import bytes_to_string
 class Model(_AbstractModel):
     """
     The section in the store that saves rows of the same kind
+
+    Model has some custom fields you can set in your models that alter the behavior of how this is stored in redis
+
+    _primary_key_field -- The field of your model that is the primary key
+    _redis_prefix -- If set, will be added to the beginning of the keys we store in redis
+    _redis_separator -- Defaults to :, used to separate prefix, table_name, and primary_key
+    _table_name -- Defaults to the model's name, can set a custom name in redis
+
+
+    If your model was named ThisModel, the primary key was "key", and prefix and separator were left at default (not set), the
+    keys stored in redis would be
+    thismodel:key
     """
 
+    _auto_sync = True
+
     @classmethod
+    @lru_cache(1)
+    def _get_prefix(cls) -> str:
+        prefix_str = getattr(cls, "_redis_prefix", "").lower()
+        return f"{prefix_str}{cls._get_separator()}" if prefix_str != "" else ""
+
+    @classmethod
+    @lru_cache(1)
+    def _get_separator(cls):
+        return getattr(cls, "_redis_separator", ":").lower()
+
+    @classmethod
+    @lru_cache(1)
+    def _get_tablename(cls):
+        return cls.__name__.lower() if cls._table_name is None else cls._table_name
+
+    @classmethod
+    @lru_cache(1)
     def __get_primary_key(cls, primary_key_value: Any):
         """
-        Returns the primary key value concatenated to the table name for uniqueness
+        Uses _table_name, _table_refix, and _redis_separator from the model to build our primary key.
+
+        _table_name defaults to the name of the model class if it is not set
+        _redis_separator defualts to : if it is not set
+        _prefix defaults to nothing if it is not set
+
+        The key is contructed as {_prefix}{_redis_separator}{_table_name}{_redis_separator}{primary_key_value}
+        So a model named ThisModel with a primary key of id, by default would result in a key of thismodel:id
         """
-        table_name = (
-            cls.__name__.lower() if cls._table_name is None else cls._table_name
-        )
-        return f"{table_name}_%&_{primary_key_value}"
+
+        return f"{cls._get_prefix()}{cls._get_tablename()}{cls._get_separator()}{primary_key_value}"
 
     @classmethod
     def get_table_index_key(cls):
         """Returns the key in which the primary keys of the given table have been saved"""
-        table_name = (
-            cls.__name__.lower() if cls._table_name is None else cls._table_name
-        )
-        return f"{table_name}__index"
+        return f"{cls._get_prefix()}{cls._get_tablename()}{cls._get_separator()}__index"
 
     @classmethod
     async def _ids_to_primary_keys(
@@ -88,32 +124,26 @@ class Model(_AbstractModel):
 
         return response
 
-    @classmethod
-    async def update(
-        cls, _id: Any, data: Dict[str, Any], life_span_seconds: Optional[int] = None
-    ):
-        """
-        Updates a given row or sets of rows in the table
-        """
-        life_span = (
-            life_span_seconds
-            if life_span_seconds is not None
-            else cls._store.life_span_in_seconds
-        )
-        async with cls._store.redis_store.pipeline(transaction=True) as pipeline:
+    def __setattr__(self, name: str, value: Any):
+        super().__setattr__(name, value)
+        store = getattr(self, "_store", None)
+        if self._auto_sync and store is not None:
+            if version_info.major == 3 and version_info.minor < 10:
+                # less than 3.10.0
+                io_loop = asyncio.get_event_loop()
+            else:
+                # equal or greater than 3.10.0
+                try:
+                    io_loop = asyncio.get_running_loop()
+                except RuntimeError:
+                    io_loop = asyncio.new_event_loop()
+            # https://github.com/erdewit/nest_asyncio
+            # Use nest_asyncio so we can call the async save
+            nest_asyncio.apply()
+            io_loop.run_until_complete(self.save())
 
-            if isinstance(data, dict):
-                name = cls.__get_primary_key(primary_key_value=_id)
-                pipeline.hset(name=name, mapping=cls.serialize_partially(data))
-                if life_span is not None:
-                    pipeline.expire(name=name, time=life_span)
-                # save the primary key in an index
-                table_index_key = cls.get_table_index_key()
-                pipeline.sadd(table_index_key, name)
-                if life_span is not None:
-                    pipeline.expire(table_index_key, time=life_span)
-            response = await pipeline.execute()
-        return response
+    async def save(self):
+        await self.insert(self)
 
     @classmethod
     async def delete(
