@@ -3,10 +3,10 @@ from typing import Any
 from typing import Dict
 from typing import Optional
 
-import aioredis
 from pydantic_aioredis.abstract import _AbstractStore
 from pydantic_aioredis.config import RedisConfig
 from pydantic_aioredis.model import Model
+from redis import asyncio as aioredis
 
 
 class Store(_AbstractStore):
